import time
from .stream import send_time

class Profiler(object):
    def __init__(self, metric):
        self.metric = metric
        self.start_stamp = None

    def __enter__(self):
        self.start_stamp = time.time()

    def __exit__(self, exc_type, exc_val, exc_tb):
        duration = (time.time() - self.start_stamp) * 1000  # msec

        send_time(self.metric,
                  duration)



import time
import functools

from .stream import send_time

def measure(prefix=None, name=None):
    def _timing(func):
        @functools.wraps(func)
        def _wrapper(*args, **kwargs):
            metric = name or ('%s.%s' % (func.__module__,
                                         func.__name__))
            if prefix:
                metric = '%s.%s' % (prefix, metric)
            start = time.time()
            result = func(*args, **kwargs)
            duration = (time.time() - start) * 1000  # msec
            send_time(metric, duration)
            return result

        return _wrapper

    return _timing



from .conf import settings

from .stream import send
from .measure import *
from .decorators import *
from .profile import *


def configure(parent):
    """
    Bind settings to upstream application main config
    """
    settings.set_parent(parent)



import time
from .stream import send_time

class MeasureMiddleware(object):

    def process_view(self, request, view_func, view_args, view_kwargs):
        request._cc_start_time = time.time()
        request._cc_view_name = '%s.%s' % (view_func.__module__,
                                           view_func.__name__)

    def process_response(self, request, response):
        if hasattr(request, '_cc_start_time'):
            duration = (time.time() - request._cc_start_time) * 1000  # msec

            metric = 'view.%s.%s' % (request._cc_view_name,
                                     request.method)
            send_time(metric, duration)


        return response



import socket
import time
from .conf import settings

_carbon_socket = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
_host_name = socket.gethostname()

def get_hostname():
    return settings.CARBON_HOSTNAME or _host_name

def row_format(hostame, metric, value, stamp):
    full_metric = "%s.%s" % (hostame, metric)
    if settings.CARBON_PREFIX:
        full_metric = "%s.%s" % (settings.CARBON_PREFIX, full_metric)

    return "%s %d %d" % (full_metric, value, stamp)


def send_buff(port, buff):
    msg = "\n".join(buff)
    msg += "\n"
    _carbon_socket.sendto(msg,
                          (settings.CARBON_SERVER,
                           port))

def send(metric, value, stamp=None, aggregator=False):
    stamp = stamp or time.time()

    buff = [ row_format(get_hostname(),
                        metric, value,
                        stamp) ]

    if settings.CARBON_SEND_ALL:
        buff.append(row_format('all',
                               metric, value,
                               stamp))
    if aggregator and settings.CARBON_AGGREGATOR_PORT:
        port = settings.CARBON_AGGREGATOR_PORT
    else:
        port = settings.CARBON_PORT
    send_buff(port, buff)


def send_time(metric, value, stamp=None):
    stamp = stamp or time.time()
    buff = []
    buff.append(row_format(get_hostname(),
                           metric+'.time',
                           value,
                           stamp))

    if settings.CARBON_SEND_ALL:
        buff.append(row_format('all',
                               metric+'.time',
                               value,
                               stamp))

    if settings.CARBON_SEND_COUNT:
        buff.append(row_format(get_hostname(),
                               metric+'.count',
                               1,
                               stamp))

    if settings.CARBON_SEND_COUNT and settings.CARBON_SEND_ALL:
        buff.append(row_format('all',
                               metric+'.count',
                               1,
                               stamp))
    send_buff(settings.CARBON_AGGREGATOR_PORT or settings.CARBON_PORT,
              buff)



"""
Carbon settings options:

CARBON_HOST            - target address where statistics will be send
CARBON_PORT            - carbon server port, defaylt is 2003
CARBON_AGGREGATOR_PORT - port of the carbon aggregator,
                         None mean no aggregator
CARBON_HOSTNAME        - source host name, None trigger hostname resolve
CARBON_PREFIX          - global metrica prefix
CARBON_SEND_ALL        - send additional metric where hostname
                         replaced with 'all'
CARBON_SEND_COUNT      - send extra metic with .count suffix and 1 value

"""

class ConfiManager(object):
    """
    Proxy config manager
    """
    DEFAULT = {
        'CARBON_HOST': 'localhost',
        'CARBON_PORT': 2003,
        'CARBON_AGGREGATOR_PORT': None,
        'CARBON_HOSTNAME': None,
        'CARBON_PREFIX': None,
        'CARBON_SEND_ALL': False,
        'CARBON_SEND_COUNT': False,
    }

    def __init__(self):
        self.parent = object()

    def __getattr__(self, key):
        if hasattr(self.parent, key):
            return getattr(self.parent, key)
        return self.DEFAULT.get(key)

    def set_parent(self, parent):
        self.parent = parent


settings = ConfiManager()



import time
import functools

from .stream import send_time

def measure(prefix, name=None):
    """Decorator for send stats to graphite"""
    def _timing(func):

        @functools.wraps(func)
        def _wrapper(*args, **kwargs):
            _name = name or ('%s.%s' % (func.__module__,
                                        func.__name__))
            metric = '%s.%s' % (prefix, _name)
            start = time.time()
            result = func(*args, **kwargs)
            duration = (time.time() - start) * 1000  # msec

            send_time(metric, duration)
            return result

        return _wrapper

    return _timing
